@@ -128,7 +128,7 @@ class Fplot:
         set data for plotting
         > gfx = Fplot(kind='scatter2D')
         > gfx.plot(x,y,label='y=f(x)').show()
-        x,y are numpy 1D arrays
+        x,y are numpy 1D arrays same size
         """
         try:
             g = self.__init
@@ -137,7 +137,11 @@ class Fplot:
         assert self.__kind == Fplot.__G2D,f"use plot only if u want draw 2D gfx, change ur kind type to: {Fplot.__G2D}"
         assert isinstance(x,(np.ndarray,list)),"x must be numpy array or list"
         assert isinstance(y,(np.ndarray,list)),"y must be numpy array or list"
-        assert len(x)==len(y),"arrays must have same lenght"
+        #assert len(x)==len(y),"arrays must have same lenght"
+        if len(x)>len(y):
+            x = x[:len(y)]
+        if len(y)>len(x):
+            y = y[:len(x)]
         self.__check_assert(typ='xyz',**d)
         datf = pd.DataFrame({'x':x,'y':y})
         datf.info = label
