@@ -155,34 +155,77 @@ class TestFunctions():
                     resultat = False  
         return resultat
 
+    # def check_op2(self,df,y,func,*x,**d):
+    #     # fonction qui vérifie qu'une colonne est une fonction de plusieurs autres
+    #     # df : dataframe
+    #     # y colonne à évaluer
+    #     # func : fonction lambda crypté de plusieurs variables
+    #     #*x colonnes 
+    #     #local_args : dict d'autres grandeurs qui ne sont pas des colonnes
+    #     k = d.get('ñ',0)
+    #     local_args =d.get("local_args",None) #de la forma {"A_f":0.90,...} extra valeurs
+    #     if k == TestFunctions.__SPY_KEY:
+    #         print("la méthode check_op ne fonctionne pas quand elle est décorée")
+    #         exp_u = TestFunctions.__f_crypt(func,TestFunctions.__SPY_KEY,"crypt")
+    #         print(func,exp_u)
+    #         (_,_,_,text)=traceback.extract_stack()[-2]
+    #         print(text)
+    #         df_name = text[text.index('_op2(')+5:text.index(',')].strip()
+    #         txt = f"{self.__def_name}.check_op2({df_name},'{y}','{exp_u}',{','.join(list(x))})"
+    #         return None
+    #     func_txt = TestFunctions.__f_crypt(func,TestFunctions.__SPY_KEY,"decrypt")
+    #     funcd = eval(func_txt)
+    #     txt, args_count = [],0
+    #     for arg in x:
+    #         txt.append(f"""df['{arg}']""")
+    #         args_count += 1
+    #     if local_args is not None:
+    #         for arg in local_args:
+    #             txt.append(f"""{local_args[arg]}""")
+    #             args_count += 1
+    #     txt = ",".join(txt)
+    #     try:
+    #         if args_count == 1:
+    #             df[f"__eval_{y}"] = funcd(eval(txt))
+    #         else:
+    #             df[f"__eval_{y}"] = funcd(*eval(txt))
+    #         resultat = np.isclose(df[y].fillna(0),df[f"__eval_{y}"].fillna(0))
+    #         #☻print(resultat)
+    #     except:
+    #         print(f"Rien ne va, votre crypto est mauvais\n{funcd}")
+    #         return None
+    #     final_result = False
+    #     if resultat.all():
+    #         final_result = True
+    #         print(f"La fonction qui définie {y} convient -> {Back.GREEN} Le test est réussi"+Style.RESET_ALL)
+    #     else:
+    #         print(f"La fonction qui définie {y} ne convient pas -> {Back.RED} echec"+Style.RESET_ALL)
+    #     df.del_columns(f"__eval_{y}")
+    #     return final_result
+    
     def check_op2(self,df,y,func,*x,**d):
-        # fonction qui vérifie qu'une colonne est une fonction de plusieurs autres
-        # df : dataframe
-        # y colonne à évaluer
-        # func : fonction lambda crypté de plusieurs variables
-        #*x colonnes 
-        #local_args : dict d'autres grandeurs qui ne sont pas des colonnes
+    # fonction qui vérifie qu'une colonne est une fonction de plusieurs autres
+    # df : dataframe
+    # y colonne à évaluer
+    # func : fonction lambda crypté de plusieurs variables
+    #*x colonnes ou extra args
         k = d.get('ñ',0)
-        local_args =d.get("local_args",None) #de la forma {"A_f":0.90,...} extra valeurs
         if k == TestFunctions.__SPY_KEY:
-            print("la méthode check_op ne fonctionne pas quand elle est décorée")
+            print("la méthode check_op2 ne fonctionne pas quand elle est décorée")
             exp_u = TestFunctions.__f_crypt(func,TestFunctions.__SPY_KEY,"crypt")
             print(func,exp_u)
             (_,_,_,text)=traceback.extract_stack()[-2]
-            print(text)
             df_name = text[text.index('_op2(')+5:text.index(',')].strip()
-            txt = f"{self.__def_name}.check_op2({df_name},'{y}','{exp_u}',{','.join(list(x))})"
             return None
         func_txt = TestFunctions.__f_crypt(func,TestFunctions.__SPY_KEY,"decrypt")
         funcd = eval(func_txt)
         txt, args_count = [],0
         for arg in x:
-            txt.append(f"""df['{arg}']""")
+            if arg in df.columns.to_list():
+                txt.append(f"""df['{arg}']""")
+            else:
+                txt.append(f"""{arg}""")
             args_count += 1
-        if local_args is not None:
-            for arg in local_args:
-                txt.append(f"""{local_args[arg]}""")
-                args_count += 1
         txt = ",".join(txt)
         try:
             if args_count == 1:
